/*
 * Copyright 2022 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef CPP_QUANTIZE_CELEBI_H_
#define CPP_QUANTIZE_CELEBI_H_

#include <stdint.h>
#include <stdlib.h>

#include <vector>

#include "cpp/quantize/wsmeans.h"
#include "cpp/utils/utils.h"

namespace material_color_utilities {

QuantizerResult QuantizeCelebi(const std::vector<Argb>& pixels,
                               uint16_t max_colors);

}  // namespace material_color_utilities

#endif  // CPP_QUANTIZE_CELEBI_H_
